﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2015 Joseph Helm and Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion


using HeuristicLab.Core;
using HeuristicLab.Operators;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Common;
using HeuristicLab.Parameters;
using HeuristicLab.Data;
using HeuristicLab.Collections;

namespace HeuristicLab.Problems.BinPacking {
  [Item("Packingplan Evaluator", "Represents a evaluator class for standard bin-packing problems.")]
  [StorableClass]
  public abstract class EvaluatorBase<D,B,I> : SingleSuccessorOperator, IEvaluator
    where D : class, IPackingPosition
    where B : PackingShape<D>
    where I : PackingShape<D>, IPackingItem {

    #region Parameters
    public ILookupParameter<IRandom> RandomParameter {
      get { return (LookupParameter<IRandom>)Parameters["Random"]; }
    }
    public ILookupParameter<DoubleValue> QualityParameter {
      get {
        if (Parameters.ContainsKey("Quality"))
          return (ILookupParameter<DoubleValue>)Parameters["Quality"];
        else
          return null;
      }
    }
    public ILookupParameter<IPackingPlan> PackingPlanParameter {
      get { return (ILookupParameter<IPackingPlan>)Parameters["PackingPlan"]; }
    }
    public ILookupParameter<ItemList<I>> PackingItemMeasuresParameter {
      get { return (ILookupParameter<ItemList<I>>)Parameters["PackingItemMeasures"]; }
    }
    public ILookupParameter<B> PackingBinMeasuresParameter {
      get { return (ILookupParameter<B>)Parameters["PackingBinMeasures"]; }
    }
    #endregion

    [StorableConstructor]
    protected EvaluatorBase(bool deserializing) : base(deserializing) { }
    protected EvaluatorBase(EvaluatorBase<D,B,I> original, Cloner cloner)
      : base(original, cloner) {
    }
    public EvaluatorBase()
      : base() {
      Parameters.Add(new LookupParameter<DoubleValue>("Quality", "The quality value of the solution."));
      Parameters.Add(new LookupParameter<IPackingPlan>("PackingPlan", "The decoded bin-packing solution represented as generalized packing-plan."));
      Parameters.Add(new LookupParameter<IRandom>("Random", "The pseudo random number generator."));
      Parameters.Add(new LookupParameter<ItemList<I>>("PackingItemMeasures", "Packing-item data taken from the bin-packing problem-instance."));
      Parameters.Add(new LookupParameter<B>("PackingBinMeasures", "Packing-bin data taken from the bin-packing problem-instance."));

    }

    protected abstract DoubleValue Evaluate();                                                                        
    public override IOperation Apply() {
      QualityParameter.ActualValue = Evaluate();
      return base.Apply();
    }
  }
}
